@@ -34,6 +34,8 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
+#include <libpanel-util/panel-error.h>
+
 #include "nothing.h"
 #include "panel-util.h"
 #include "panel.h"
@@ -131,6 +133,7 @@ panel_context_menu_show_about_dialog (GtkWidget *menuitem)
 				    "applications and provides useful utilities."),
 		      "authors", authors,
 		      "documenters", documenters,
+		      "title", _("About the GNOME Panel"),
 		      "translator-credits", _("translator-credits"),
 		      "logo-icon-name", PANEL_ICON_PANEL,
 		      NULL);
@@ -273,6 +276,7 @@ panel_context_menu_create (PanelWidget *panel)
 	}
 
 	retval = create_empty_menu ();
+	gtk_widget_set_name (retval, "gnome-panel-context-menu");
 
 	if (!panel_lockdown_get_locked_down ())
 		panel_context_menu_build_edition (panel, retval);
