@@ -32,6 +32,8 @@ typedef enum {
 	PANEL_PUSH_MOVE
 } PanelMovementType;
 
+G_DEFINE_TYPE (PanelWidget, panel_widget, GTK_TYPE_FIXED);
+
 enum {
 	SIZE_CHANGE_SIGNAL,
 	BACK_CHANGE_SIGNAL,
@@ -55,8 +57,6 @@ static guint panel_widget_signals [LAST_SIGNAL] = {0};
 static gboolean panel_applet_in_drag = FALSE;
 static GtkWidget *saved_focus_widget = NULL;
 
-static void panel_widget_class_init     (PanelWidgetClass *klass);
-static void panel_widget_instance_init  (PanelWidget      *panel_widget);
 static void panel_widget_size_request   (GtkWidget        *widget,
 					 GtkRequisition   *requisition);
 static void panel_widget_size_allocate  (GtkWidget        *widget,
@@ -118,33 +118,6 @@ emit_applet_moved (PanelWidget *panel_widget,
  widget core
  ************************/
 
-static GtkFixedClass *panel_widget_parent_class = NULL;
-
-GType
-panel_widget_get_type (void)
-{
-	static GType object_type= 0;
-
-	if (object_type == 0) {
-		static const GTypeInfo object_info = {
-                    sizeof (PanelWidgetClass),
-                    (GBaseInitFunc)         NULL,
-                    (GBaseFinalizeFunc)     NULL,
-                    (GClassInitFunc)        panel_widget_class_init,
-                    NULL,                   /* class_finalize */
-                    NULL,                   /* class_data */
-                    sizeof (PanelWidget),
-                    0,                      /* n_preallocs */
-                    (GInstanceInitFunc)     panel_widget_instance_init 
-		};
-
-		object_type = g_type_register_static (GTK_TYPE_FIXED , "PanelWidget", &object_info, 0);
-		panel_widget_parent_class = g_type_class_ref (GTK_TYPE_FIXED);
-	}
-
-	return object_type;
-}
-
 static void
 add_tab_bindings (GtkBindingSet    *binding_set,
    	          GdkModifierType   modifiers,
@@ -1218,6 +1191,8 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 {
 	PanelWidget *panel;
 	GList *list;
+	GList *ad_with_hints;
+	gboolean dont_fill;
 
 	g_return_if_fail(PANEL_IS_WIDGET(widget));
 	g_return_if_fail(requisition!=NULL);
@@ -1232,6 +1207,8 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 		requisition->width = panel->sz;
 	}
 
+	ad_with_hints = NULL;
+
 	for(list = panel->applet_list; list!=NULL; list = g_list_next(list)) {
 		AppletData *ad = list->data;
 		GtkRequisition chreq;
@@ -1242,7 +1219,8 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 				requisition->height = chreq.height;
 
 			if (panel->packed && ad->expand_major && ad->size_hints)
-				requisition->width += MAX (ad->size_hints [0], chreq.width); 
+				ad_with_hints = g_list_prepend (ad_with_hints,
+								ad);
 
 			else if (panel->packed)
 				requisition->width += chreq.width;
@@ -1251,25 +1229,67 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 				requisition->width = chreq.width;
 
 			if (panel->packed && ad->expand_major && ad->size_hints)
-				requisition->height += MAX (ad->size_hints [0], chreq.height);
+				ad_with_hints = g_list_prepend (ad_with_hints,
+								ad);
 
 			else if (panel->packed)
 				requisition->height += chreq.height;
 		}
 	}
 
+
+	panel->nb_applets_size_hints = 0;
+	if (panel->applets_hints != NULL)
+		g_free (panel->applets_hints);
+	panel->applets_hints = NULL;
+	if (panel->applets_using_hint != NULL)
+		g_free (panel->applets_using_hint);
+	panel->applets_using_hint = NULL;
+
 	if(!panel->packed) {
 		if(panel->orient == GTK_ORIENTATION_HORIZONTAL) {
 			requisition->width = panel->size;
 		} else {
 			requisition->height = panel->size;
 		}
-	} 
+	} else {
+		/* put the list in the correct order: this is important
+		 * since we'll use this order in the size_allocate() */
+		ad_with_hints = g_list_reverse (ad_with_hints);
+
+		panel->nb_applets_size_hints = g_list_length (ad_with_hints);
+		if (panel->nb_applets_size_hints > 0) {
+			int i;
+			panel->applets_hints = g_new0 (AppletSizeHints, panel->nb_applets_size_hints);
+
+			i = 0;
+			for (list = ad_with_hints;
+			     list != NULL;
+			     list = g_list_next (list)) {
+				AppletData *ad = list->data;
+
+				panel->applets_hints[i].hints = ad->size_hints;
+				panel->applets_hints[i].len = ad->size_hints_len;
+				i++;
+			}
+
+			panel->applets_using_hint = g_new0 (AppletSizeHintsAlloc, panel->nb_applets_size_hints);
+		}
+	}
 	
-	if (requisition->width < 12)
-		requisition->width = 12;
-	if (requisition->height < 12)
-		requisition->height = 12;
+	dont_fill = panel->packed && panel->nb_applets_size_hints != 0;
+
+	if (panel->orient == GTK_ORIENTATION_HORIZONTAL) {
+		if (requisition->width < 12 && !dont_fill)
+			requisition->width = 12;
+		if (requisition->height < 12)
+			requisition->height = 12;
+	} else {
+		if (requisition->width < 12)
+			requisition->width = 12;
+		if (requisition->height < 12 && !dont_fill)
+			requisition->height = 12;
+	}
 }
 
 static void
@@ -1303,22 +1323,6 @@ panel_widget_set_background_region (PanelWidget *panel)
 		widget->allocation.height);
 }
 
-static int
-panel_widget_count_expanded_applets (PanelWidget *panel)
-{
-	GList *li;
-	int count = 0;
-
-	for (li = panel->applet_list; li != NULL; li = li->next) {
-		AppletData *ad = li->data;
-
-		if (ad->expand_major)
-			count ++;
-	}
-
-	return count;
-}
-
 static void
 panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -1326,6 +1330,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	GList *list;
 	int i;
 	int old_size;
+	gboolean ltr;
 
 	g_return_if_fail(PANEL_IS_WIDGET(widget));
 	g_return_if_fail(allocation!=NULL);
@@ -1333,6 +1338,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	panel = PANEL_WIDGET(widget);
 
 	old_size = panel->size;
+	ltr = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR;
 	
 	widget->allocation = *allocation;
 	if (GTK_WIDGET_REALIZED (widget))
@@ -1350,20 +1356,9 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 		panel_widget_right_stick(panel,old_size);
 
 	if (panel->packed) {
-		int compression = 0;
-		int applets_to_compress = 0;
-
-		if (panel->orient == GTK_ORIENTATION_HORIZONTAL &&
-		    widget->requisition.width > allocation->width) {
-			compression = widget->requisition.width - allocation->width;
-		} else if (panel->orient == GTK_ORIENTATION_VERTICAL &&
-			   widget->requisition.height > allocation->height) {
-			compression = widget->requisition.height - allocation->height;
-		}
-
-		if (compression > 0) {
-			applets_to_compress = panel_widget_count_expanded_applets (panel);
-		}
+		/* we're assuming the order is the same as the one that was
+		 * in size_request() */
+		int applet_using_hint_index = 0;
 
 		i = 0;
 		for(list = panel->applet_list;
@@ -1383,34 +1378,22 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 					challoc.height = allocation->height;
 
 				if (ad->expand_major && ad->size_hints) {
-					int width = ad->size_hints [0];
-					if (applets_to_compress > 0) {
-						width -= (compression / applets_to_compress);
-						compression -= (compression / applets_to_compress);
-						applets_to_compress --;
-					}
-					challoc.width = CLAMP (MAX (width, chreq.width),
-							       chreq.width,
-							       allocation->width - i);
+					int width = panel->applets_using_hint[applet_using_hint_index].size;
+					applet_using_hint_index++;
+					challoc.width = MIN (width, allocation->width - i);
 				}
 
 				ad->cells = challoc.width;
-				challoc.x = ad->constrained;
+				challoc.x = ltr ? ad->constrained : panel->size - ad->constrained - challoc.width;
 				challoc.y = allocation->height / 2 - challoc.height / 2;
 			} else {
 				if (ad->expand_minor)
 					challoc.width = allocation->width;
 
 				if (ad->expand_major && ad->size_hints) {
-					int height = ad->size_hints [0];
-					if (applets_to_compress > 0) {
-						height -= (compression / applets_to_compress);
-						compression -= (compression / applets_to_compress);
-						applets_to_compress --;
-					}
-					challoc.height = CLAMP (MAX (height, chreq.height),
-								chreq.height,
-								allocation->height - i);
+					int height = panel->applets_using_hint[applet_using_hint_index].size;
+					applet_using_hint_index++;
+					challoc.height = MIN (height, allocation->height - i);
 				}
 
 				ad->cells = challoc.height;
@@ -1421,6 +1404,10 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			gtk_widget_size_allocate(ad->applet,&challoc);
 			i += ad->cells;
 		}
+
+		/* EEEEK, there might be not enough room and we don't handle
+		 * it: all the applets at the right well be unusable */
+
 	} else { /*not packed*/
 
 		/* First make sure there's enough room on the left */
@@ -1433,12 +1420,17 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
 			gtk_widget_get_child_requisition(ad->applet,&chreq);
 
-			if(panel->orient == GTK_ORIENTATION_HORIZONTAL)
-				ad->cells = chreq.width;
-			else
-				ad->cells = chreq.height;
+			if (!ad->expand_major || !ad->size_hints) {
+				if(panel->orient == GTK_ORIENTATION_HORIZONTAL)
+					ad->cells = chreq.width;
+				else
+					ad->cells = chreq.height;
 
-			ad->min_cells = ad->cells;
+				ad->min_cells = ad->cells;
+			} else {
+				ad->cells = ad->size_hints [ad->size_hints_len - 1];
+				ad->min_cells = ad->size_hints [ad->size_hints_len - 1];
+			}
 
 			ad->constrained = ad->pos;
 
@@ -1457,7 +1449,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			int cells;
 			
 			if (ad->constrained + ad->min_cells > i)
-				ad->constrained = i - ad->min_cells;
+				ad->constrained = MAX (i - ad->min_cells, 0);
 			
 			if (ad->expand_major) {
 				cells = (i - ad->constrained) - 1;
@@ -1465,6 +1457,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				if (ad->size_hints) 
 					cells = get_size_from_hints (ad, cells);
 				cells = MAX (cells, ad->min_cells);
+				cells = MIN (cells, panel->size);
 
 				ad->cells = cells;
 			}
@@ -1503,7 +1496,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				if (ad->expand_minor) {
 					challoc.height = allocation->height;
 				}
-				challoc.x = ad->constrained;
+				challoc.x = ltr ? ad->constrained : panel->size - ad->constrained - challoc.width;
 				challoc.y = allocation->height / 2 - challoc.height / 2;
 			} else {
 				challoc.height = ad->cells;
@@ -1629,6 +1622,14 @@ panel_widget_finalize (GObject *obj)
 
 	panel_background_free (&panel->background);
 
+	if (panel->applets_hints != NULL)
+		g_free (panel->applets_hints);
+	panel->applets_hints = NULL;
+	if (panel->applets_using_hint != NULL)
+		g_free (panel->applets_using_hint);
+	panel->applets_using_hint = NULL;
+
+
 	G_OBJECT_CLASS (panel_widget_parent_class)->finalize (obj);
 }
 
@@ -1686,7 +1687,7 @@ panel_widget_destroy (GtkObject *obj)
 }
 
 static void
-panel_widget_instance_init (PanelWidget *panel)
+panel_widget_init (PanelWidget *panel)
 {
 	GtkWidget *widget = (GtkWidget *) panel;
 
@@ -1703,6 +1704,10 @@ panel_widget_instance_init (PanelWidget *panel)
 	panel->drop_widget   = widget;
 	panel->open_dialogs  = NULL;
 
+	panel->nb_applets_size_hints = 0;
+	panel->applets_hints = NULL;
+	panel->applets_using_hint = NULL;
+
 	panel_background_init (&panel->background,
 			       (PanelBackgroundChangedNotify) panel_widget_background_changed,
 			       panel);
@@ -1867,17 +1872,22 @@ panel_widget_applet_drag_end (PanelWidget *panel)
 	gdk_flush ();
 }
 
-/*get pos of the cursor location*/
+/*get pos of the cursor location in panel coordinates*/
 int
 panel_widget_get_cursorloc (PanelWidget *panel)
 {
 	int x, y;
+	gboolean rtl;
 
 	g_return_val_if_fail (PANEL_IS_WIDGET (panel), -1);
 
 	gtk_widget_get_pointer (GTK_WIDGET (panel), &x, &y);
-
-	return panel->orient == GTK_ORIENTATION_HORIZONTAL ? x : y;
+	rtl = gtk_widget_get_direction (GTK_WIDGET (panel)) == GTK_TEXT_DIR_RTL;
+	
+	if (panel->orient == GTK_ORIENTATION_HORIZONTAL)
+		return (rtl ? panel->size - x : x);
+	else
+		return y;
 }
 
 /*calculates the value to move the applet by*/
@@ -2538,13 +2548,13 @@ panel_widget_reparent (PanelWidget *old_panel,
 	GtkWidget *focus_widget = NULL;
 	AppletInfo* info;
 
-	g_return_val_if_fail(PANEL_IS_WIDGET(old_panel),-1);
-	g_return_val_if_fail(PANEL_IS_WIDGET(new_panel),-1);
-	g_return_val_if_fail(GTK_IS_WIDGET(applet),-1);
-	g_return_val_if_fail(pos>=0,-1);
+	g_return_val_if_fail(PANEL_IS_WIDGET(old_panel), FALSE);
+	g_return_val_if_fail(PANEL_IS_WIDGET(new_panel), FALSE);
+	g_return_val_if_fail(GTK_IS_WIDGET(applet), FALSE);
+	g_return_val_if_fail(pos>=0, FALSE);
 
 	ad = g_object_get_data (G_OBJECT (applet), PANEL_APPLET_DATA);
-	g_return_val_if_fail(ad!=NULL,-1);
+	g_return_val_if_fail(ad!=NULL, FALSE);
 
 	/* Don't try and reparent to an explicitly hidden panel,
 	 * very confusing for the user ...
@@ -2869,8 +2879,13 @@ panel_widget_set_applet_size_hints (PanelWidget *panel,
 
 	g_free (ad->size_hints);
 
-	ad->size_hints     = size_hints;
-	ad->size_hints_len = size_hints_len;
+	if (size_hints_len > 0 && (size_hints_len % 2 == 0)) {
+		ad->size_hints     = size_hints;
+		ad->size_hints_len = size_hints_len;
+	} else {
+		g_free (size_hints);
+		ad->size_hints = NULL;
+	}
 
 	gtk_widget_queue_resize (GTK_WIDGET (panel));
 }
@@ -2925,6 +2940,11 @@ void
 panel_widget_register_open_dialog (PanelWidget *panel,
 				   GtkWidget   *dialog)
 {
+	/* the window is for a panel, so it should be shown in the taskbar. See
+	 * HIG: An alert should not appear in the panel window list unless it
+	 * is, or may be, the only window shown by an application. */
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
+
 	panel->open_dialogs = g_slist_append (panel->open_dialogs,
 					      dialog);
 	
